@@ -1,26 +1,25 @@
-/* 
+/*
  * AFM library public interface.
- * Copyright (c) 1995 Markku Rossi.
+ * Copyright (c) 1995-1998 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
 
 /*
- * This file is part of the AFM library.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "afmint.h"
@@ -245,7 +244,7 @@ afm_open_file (AFMHandle handle, unsigned int info_level,
 {
   AFMFont font;
   AFMError error = AFM_SUCCESS;
-  
+
   if (handle == NULL || filename == NULL)
     return AFM_ERROR_ARGUMENT;
 
@@ -339,7 +338,7 @@ afm_close_font (AFMFont font)
       FREE (font->kern_pairs[i].name2);
     }
   FREE (font->kern_pairs);
-  
+
   /* Track kern. */
   FREE (font->track_kerns);
 
@@ -412,7 +411,7 @@ afm_font_dump (FILE *fp, AFMFont font)
       cm = &font->character_metrics[i];
 
       fprintf (fp, "  C %ld ; N %s ; B %g %g %g %g\n",
-	       cm->character_code, STR (cm->name), 
+	       cm->character_code, STR (cm->name),
 	       cm->llx, cm->lly, cm->urx, cm->ury);
       fprintf (fp,
 	       "    W0X %g ; W0Y %g ; W1X %g ; W1Y %g ; VV %g %g\n",
@@ -486,7 +485,7 @@ afm_font_stringwidth (AFMFont font, AFMNumber ptsize, char *string,
       /* Count character by character. */
       for (i = 0; i < stringlen; i++)
 	{
-	  cm = font->encoding[(unsigned int) string[i]];
+	  cm = font->encoding[(unsigned char) string[i]];
 	  if (cm == AFM_ENC_NONE || cm == AFM_ENC_NON_EXISTENT)
 	    {
 	      /* Use the undef font. */
@@ -501,7 +500,7 @@ afm_font_stringwidth (AFMFont font, AFMNumber ptsize, char *string,
 	    }
 	}
     }
-  
+
   *w0x_return = x / UNITS_PER_POINT * ptsize;
   *w0y_return = y / UNITS_PER_POINT * ptsize;
 
@@ -510,13 +509,13 @@ afm_font_stringwidth (AFMFont font, AFMNumber ptsize, char *string,
 
 
 AFMError
-afm_font_charwidth (AFMFont font, AFMNumber ptsize, unsigned int ch,
+afm_font_charwidth (AFMFont font, AFMNumber ptsize, char ch,
 		    AFMNumber *w0x_return, AFMNumber *w0y_return)
 {
   AFMNumber x = 0.0;
   AFMNumber y = 0.0;
   AFMIndividualCharacterMetrics *cm;
-  
+
   if (!font || !font->writing_direction_metrics[0].is_valid)
     return AFM_ERROR_ARGUMENT;
 
@@ -528,7 +527,7 @@ afm_font_charwidth (AFMFont font, AFMNumber ptsize, unsigned int ch,
     }
   else
     {
-      cm = font->encoding[(unsigned int) ch];
+      cm = font->encoding[(unsigned char) ch];
       if (cm == AFM_ENC_NONE || cm == AFM_ENC_NON_EXISTENT)
 	{
 	  /* Use the undef font. */
@@ -618,20 +617,28 @@ afm_font_encoding (AFMFont font, AFMEncoding enc, unsigned int flags)
 	}
       break;
 
-    case AFM_ENCODING_LATIN1:
-      apply_encoding (font, afm_latin1_encoding, flags);
+    case AFM_ENCODING_ISO_8859_1:
+      apply_encoding (font, afm_88591_encoding, flags);
       break;
 
-    case AFM_ENCODING_LATIN2:
-      apply_encoding (font, afm_latin2_encoding, flags);
+    case AFM_ENCODING_ISO_8859_2:
+      apply_encoding (font, afm_88592_encoding, flags);
       break;
 
-    case AFM_ENCODING_LATIN3:
-      apply_encoding (font, afm_latin3_encoding, flags);
+    case AFM_ENCODING_ISO_8859_3:
+      apply_encoding (font, afm_88593_encoding, flags);
       break;
 
-    case AFM_ENCODING_LATIN5:
-      apply_encoding (font, afm_latin5_encoding, flags);
+    case AFM_ENCODING_ISO_8859_4:
+      apply_encoding (font, afm_88594_encoding, flags);
+      break;
+
+    case AFM_ENCODING_ISO_8859_5:
+      apply_encoding (font, afm_88595_encoding, flags);
+      break;
+
+    case AFM_ENCODING_ISO_8859_7:
+      apply_encoding (font, afm_88597_encoding, flags);
       break;
 
     case AFM_ENCODING_IBMPC:
@@ -643,7 +650,7 @@ afm_font_encoding (AFMFont font, AFMEncoding enc, unsigned int flags)
        * First apply one encoding (all have equal first 128 characters),
        * then zap last 128 chars.
        */
-      apply_encoding (font, afm_latin1_encoding, flags);
+      apply_encoding (font, afm_88591_encoding, flags);
       for (i = 128; i < 256; i++)
 	font->encoding[i] = AFM_ENC_NONE;
       break;
@@ -675,66 +682,19 @@ afm_font_encoding (AFMFont font, AFMEncoding enc, unsigned int flags)
 
 
 void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  afm_message (AFMHandle handle, int level, char *fmt, ...)
-#else
-  afm_message (handle, level, va_alist)
-     AFMHandle handle;
-     int level;
-     va_dcl
-#endif
+afm_message (AFMHandle handle, int level, char *message)
 {
-  va_list args;
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start(args, fmt);
-#else
-  char *fmt;
-
-  va_start(args);
-  fmt = va_arg(args, char *);
-#endif
-
   if (handle->verbose < level)
     return;
 
-#ifdef HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-  _doprnt(fmt, args, stderr);
-#endif
-  va_end (args);
+  fprintf (stderr, "%s", message);
 }
 
 
 void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  afm_error (AFMHandle handle, char *fmt, ...)
-#else
-  afm_error (handle, va_alist)
-     AFMHandle handle;
-     va_dcl
-#endif
+afm_error (AFMHandle handle, char *message)
 {
-  va_list args;
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start(args, fmt);
-#else
-  char *fmt;
-
-  va_start(args);
-  fmt = va_arg(args, char *);
-#endif
-
-  fprintf (stderr, "AFM Error: ");
-
-#ifdef HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-   _doprnt(fmt, args, stderr);
-#endif
-
-  va_end (args);
-  fprintf (stderr, "\n");
+  fprintf (stderr, "AFM Error: %s\n", message);
 }
 
 
@@ -750,14 +710,17 @@ read_font_map (AFMHandle handle, char *name)
   char fullname[512];
   unsigned int dirlen;
   char *cp, *cp2;
+  char msg[256];
 
-  afm_message (handle, 1, "AFM: reading font map \"%s\"\n", name);
+  sprintf (msg, "AFM: reading font map \"%s\"\n", name);
+  afm_message (handle, 1, msg);
 
   fp = fopen (name, "r");
   if (fp == NULL)
     {
-      afm_message (handle, 1, "AFM: couldn't open font map \"%s\": %s\n",
-		   name, strerror (errno));
+      sprintf (msg, "AFM: couldn't open font map \"%s\": %s\n", name,
+	       strerror (errno));
+      afm_message (handle, 1, msg);
       return;
     }
 
@@ -781,11 +744,12 @@ read_font_map (AFMHandle handle, char *name)
 
       if (sscanf (buf, "%s %s", font, file) != 2)
 	{
-	  afm_error (handle, "malformed line in font map \"%s\":\n%s",
-		     name, buf);
+	  sprintf (msg, "malformed line in font map \"%s\":\n%s",
+		   name, buf);
+	  afm_error (handle, msg);
 	  continue;
 	}
-      
+
       /* Do we already have this font? */
       if (strhash_get (handle->font_map, font, strlen (font), (void *) &cp))
 	continue;
@@ -800,7 +764,8 @@ read_font_map (AFMHandle handle, char *name)
 	}
       strcpy (cp, fullname);
 
-      afm_message (handle, 2, "AFM: font mapping: %s -> %s\n", font, cp);
+      sprintf (msg, "AFM: font mapping: %s -> %s\n", font, cp);
+      afm_message (handle, 2, msg);
       (void) strhash_put (handle->font_map, font, strlen (font), cp,
 			  (void *) &cp2);
     }
