@@ -1,26 +1,25 @@
-/* 
+/*
  * Parse AFM files.
- * Copyright (c) 1995 Markku Rossi.
+ * Copyright (c) 1995-1998 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
 
 /*
- * This file is part of the AFM library.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "afmint.h"
@@ -49,7 +48,7 @@ typedef struct parse_ctx_st ParseCtx;
  * Static variables.
  */
 
-/* 
+/*
  * The AFM keys.  This array must be kept sorted because keys are
  * searched by using binary search.
  */
@@ -193,7 +192,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
     parse_error (handle, AFM_ERROR_NOT_AFM_FILE);
   GET_VALUE (AFM_TYPE_NUMBER);
   font->version = node.u.number;
-    
+
   /* Parse it. */
   while (!done)
     {
@@ -215,7 +214,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	case kEndFontMetrics:
 	  done = 1;
 	  break;
-	  
+
 	case kStartCompFontMetrics:
 	case kEndCompFontMetrics:
 	case kStartMasterFontMetrics:
@@ -254,7 +253,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  font->global_info.FontBBox_ury = node.u.number;
 	  break;
-	  
+
 	case kVersion:
 	  GET_VALUE (AFM_TYPE_STRING);
 	  font->global_info.Version = node.u.string;
@@ -468,7 +467,7 @@ afm_parse_file (AFMHandle handle, const char *filename, AFMFont font)
   fclose (ctx->fp);
 
   /* Check post conditions. */
-  
+
   if (!font->writing_direction_metrics[0].is_valid
       && !font->writing_direction_metrics[1].is_valid)
     /* No direction specified, 0 implied. */
@@ -545,7 +544,7 @@ get_token (AFMHandle handle, ParseCtx *ctx)
   if (i >= sizeof (ctx->token))
     /* Line is too long, this is against AFM specification. */
     parse_error (handle, AFM_ERROR_SYNTAX);
-  
+
   ctx->token[i] = '\0';
   ctx->tokenlen = i;
 
@@ -557,7 +556,7 @@ static int
 get_line_token (AFMHandle handle, ParseCtx *ctx)
 {
   int i, ch;
-  
+
   /* Skip the leading whitespace. */
   while ((ch = getc (ctx->fp)) != EOF)
     if (!ISSPACE (ch))
@@ -602,7 +601,7 @@ match_key (char *key)
       midpoint = (lower + upper) / 2;
       if (keynames[midpoint].name == NULL)
 	break;
-      
+
       cmpvalue = strcmp (key, keynames[midpoint].name);
       if (cmpvalue == 0)
 	found = AFMTrue;
@@ -623,6 +622,7 @@ static void
 get_key (AFMHandle handle, ParseCtx *ctx, AFMKey *key_return)
 {
   int key;
+  char msg[256];
 
   while (1)
     {
@@ -638,7 +638,8 @@ get_key (AFMHandle handle, ParseCtx *ctx, AFMKey *key_return)
 	}
 
       /* No match found.  According to standard, we must skip this key. */
-      afm_error (handle, "skipping key \"%s\"", ctx->token);
+      sprintf (msg, "skipping key \"%s\"", ctx->token);
+      afm_error (handle, msg);
       get_line_token (handle, ctx);
     }
 
@@ -796,14 +797,14 @@ read_character_metrics (AFMHandle handle, ParseCtx *ctx, AFMFont font)
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->w1y = node.u.number;
 	  break;
-	      
+
 	case kVV:
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->vv_x = node.u.number;
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->vv_y = node.u.number;
 	  break;
-	      
+
 	case kN:
 	  GET_VALUE (AFM_TYPE_NAME);
 	  cm->name = node.u.name;
@@ -811,7 +812,7 @@ read_character_metrics (AFMHandle handle, ParseCtx *ctx, AFMFont font)
 			    strlen (cm->name), cm, NULL))
 	    parse_error (handle, AFM_ERROR_MEMORY);
 	  break;
-	      
+
 	case kB:
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->llx = node.u.number;
@@ -822,7 +823,7 @@ read_character_metrics (AFMHandle handle, ParseCtx *ctx, AFMFont font)
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->ury = node.u.number;
 	  break;
-	      
+
 	case kL:
 	  /* XXX Skip ligatures. */
 	  get_line_token (handle, ctx);
@@ -842,7 +843,7 @@ read_character_metrics (AFMHandle handle, ParseCtx *ctx, AFMFont font)
 
 	  done = 1;
 	  break;
-	      
+
 	default:
 	  parse_error (handle, AFM_ERROR_SYNTAX);
 	  break;
@@ -1002,7 +1003,7 @@ read_composites (AFMHandle handle, ParseCtx *ctx, AFMFont font)
 
 	  GET_VALUE (AFM_TYPE_NAME);
 	  cm->components[j].name = node.u.name;
-	  
+
 	  GET_VALUE (AFM_TYPE_NUMBER);
 	  cm->components[j].deltax = node.u.number;
 
