@@ -1,13 +1,13 @@
-/* 
+/*
  * Internal header file.
- * Copyright (c) 1995 Markku Rossi.
+ * Copyright (c) 1995-1998 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -52,7 +52,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
 
 #else /* no STDC_HEADERS */
 
@@ -71,12 +70,6 @@
 char *strchr ();
 char *strrchr ();
 
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-#include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
-
 #ifndef HAVE_STRERROR
 extern char *strerror ___P ((int));
 #endif
@@ -133,7 +126,7 @@ extern double atan2 ___P ((double, double));
 #include "strhash.h"
 #include "xalloc.h"
 
-/* 
+/*
  * Types and definitions.
  */
 
@@ -181,10 +174,12 @@ typedef enum
 
 typedef enum
 {
-  ENC_LATIN1,
-  ENC_LATIN2,
-  ENC_LATIN3,
-  ENC_LATIN5,
+  ENC_ISO_8859_1,
+  ENC_ISO_8859_2,
+  ENC_ISO_8859_3,
+  ENC_ISO_8859_4,
+  ENC_ISO_8859_5,
+  ENC_ISO_8859_7,
   ENC_ASCII,
   ENC_ASCII_FISE,
   ENC_ASCII_DKNO,
@@ -202,6 +197,14 @@ typedef enum
   LABEL_LONG
 } PageLabelFormat;
 
+typedef enum
+{
+  MWLS_NONE	= 0,
+  MWLS_PLUS	= 1,
+  MWLS_BOX	= 2,
+  MWLS_ARROW	= 3
+} MarkWrappedLinesStyle;
+
 typedef enum
 {
   NPF_SPACE,
@@ -213,9 +216,17 @@ typedef enum
 typedef enum
 {
   FORMFEED_COLUMN,
-  FORMFEED_PAGE
+  FORMFEED_PAGE,
+  FORMFEED_HCOLUMN
 } FormFeedType;
 
+typedef enum
+{
+  LE_TRUNCATE,
+  LE_CHAR_WRAP,
+  LE_WORD_WRAP
+} LineEndType;
+
 struct file_lookup_ctx_st
 {
   char name[256];
@@ -235,7 +246,9 @@ struct input_stream_st
   unsigned int data_in_buf;
   unsigned int bufpos;
   unsigned int nreads;
-  int unget_ch;
+  unsigned char *unget_ch;
+  unsigned int unget_pos;
+  unsigned int unget_alloc;
 };
 
 typedef struct input_stream_st InputStream;
@@ -279,7 +292,7 @@ struct cached_font_info_st
 typedef struct cached_font_info_st CachedFontInfo;
 
 
-/* 
+/*
  * Global variables.
  */
 
@@ -325,6 +338,8 @@ extern int d_output_w;
 extern int d_output_h;
 extern int d_output_x_margin;
 extern int d_output_y_margin;
+extern unsigned int nup_xpad;
+extern unsigned int nup_ypad;
 
 /* Document needed resources. */
 extern StringHashPtr res_fonts;
@@ -367,7 +382,7 @@ extern int verbose;
 extern int num_copies;
 extern char *title;
 extern int num_columns;
-extern int truncate_lines;
+extern LineEndType line_end;
 extern int quiet;
 extern int landscape;
 extern HeaderType header;
@@ -384,6 +399,7 @@ extern char *encoding_name;
 extern char encoding_name_buffer[];
 extern int special_escapes;
 extern int escape_char;
+extern int default_escape_char;
 extern int tabsize;
 extern double baselineskip;
 extern FontPoint ul_ptsize;
@@ -405,8 +421,11 @@ extern char *page_label_format;
 extern char page_label_format_buf[];
 extern int pass_through;
 extern int line_numbers;
+extern unsigned int start_line_number;
 extern int interpret_formfeed;
 extern NonPrintableFormat non_printable_format;
+extern MarkWrappedLinesStyle mark_wrapped_lines_style;
+extern char mark_wrapped_lines_style_name[];
 extern char *npf_name;
 extern char npf_name_buf[];
 extern int clean_7bit;
@@ -417,6 +436,7 @@ extern int page_prefeed;
 extern PageRange *page_ranges;
 extern int borders;
 extern double line_highlight_gray;
+extern double bggray;
 extern int accept_composites;
 extern FormFeedType formfeed_type;
 extern char *input_filter_stdin;
@@ -426,6 +446,7 @@ extern char *toc_fmt_string;
 extern unsigned int file_align;
 extern int slicing;
 extern unsigned int slice;
+extern char states_path[];
 extern char states_color_model[];
 extern char states_config_file[];
 extern char states_highlight_level[];
@@ -436,22 +457,43 @@ extern int nup_landscape;
 extern unsigned int nup_width;
 extern unsigned int nup_height;
 extern double nup_scale;
+extern char *output_language;
+extern int output_language_pass_through;
+extern int generate_PageSize;
+extern double horizontal_column_height;
 
 
-/* 
+/*
  * Prototypes for global functions.
  */
 
 /* Print message if <verbose> is >= <verbose_level>. */
-void message ___P ((int verbose_level, char *fmt, ...));
+#define MESSAGE(verbose_level, body)		\
+  do {						\
+    if (!quiet && verbose >= (verbose_level))	\
+      fprintf body;				\
+  } while (0)
 
 /* Report continuable error. */
-void error ___P ((char *fmt, ...));
+#define ERROR(body)			\
+  do {					\
+    fprintf (stderr, "%s: ", program);	\
+    fprintf body;			\
+    fprintf (stderr, "\n");		\
+    fflush (stderr);			\
+  } while (0)
 
 /* Report fatal error and exit with status 1.  Function never returns. */
-void fatal ___P ((char *fmt, ...));
+#define FATAL(body)			\
+  do {					\
+    fprintf (stderr, "%s: ", program);	\
+    fprintf body;			\
+    fprintf (stderr, "\n");		\
+    fflush (stderr);			\
+    exit (1);				\
+  } while (0)
 
-/* 
+/*
  * Read config file <path, name>.  Returns bool.  If function fails, error
  * is found from errno.
  */
@@ -475,7 +517,7 @@ int is_open ___P ((InputStream *is, FILE *fp, char *fname,
 void is_close ___P ((InputStream *is));
 
 /*
- * Read next character from the InputStream <is>.  Returns EOF if 
+ * Read next character from the InputStream <is>.  Returns EOF if
  * EOF was reached.
  */
 int is_getc ___P ((InputStream *is));
@@ -486,7 +528,7 @@ int is_getc ___P ((InputStream *is));
  */
 int is_ungetc ___P ((int ch, InputStream *is));
 
-/* 
+/*
  * Process single input file <fp>.  File's name is given in <fname> and
  * it is used to print headers.
  */
@@ -499,14 +541,14 @@ void add_media ___P ((char *name, int w, int h, int llx, int lly, int urx,
 /* Print a listing of missing characters. */
 void do_list_missing_characters ___P ((int *array));
 
-/* 
+/*
  * Check if file <name, suffix> exists.  Returns bool.  If function fails
  * error can be found from errno.
  */
 int file_existsp ___P ((char *name, char *suffix));
 
 /*
- * Paste file <name, suffix> to output stream.  Returns bool. If 
+ * Paste file <name, suffix> to output stream.  Returns bool. If
  * function fails, error can be found from errno.
  */
 int paste_file ___P ((char *name, char *suffix));
