@@ -1,13 +1,13 @@
-/* 
+/*
  * Help utilities.
- * Copyright (c) 1995 Markku Rossi.
+ * Copyright (c) 1995-1998 Markku Rossi.
  *
  * Author: Markku Rossi <mtr@iki.fi>
  */
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -26,6 +26,20 @@
 
 #include "gsint.h"
 
+/*
+ * Types and definitions.
+ */
+
+#define CFG_FATAL(body)						\
+  do {								\
+    fprintf (stderr, "%s:%s:%d: ", program, fname, line);	\
+    fprintf body;						\
+    fprintf (stderr, "\n");					\
+    fflush (stderr);						\
+    exit (1);							\
+  } while (0)
+
+
 /*
  * Static variables.
  */
@@ -69,121 +83,16 @@ static struct
 };
 
 
-/*
- * Prototypes for static functions.
- */
-
-static void cfg_fatal ___P ((char *file, int line, char *fmt, ...));
-
-
 /*
  * Global functions.
  */
 
-void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  message (int verbose_level, char *fmt, ...)
-#else
-  message (verbose_level, va_alist)
-     int verbose_level;
-     va_dcl
-#endif
-{
-  va_list args;
-
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start (args, fmt);
-#else
-  char *fmt;
-
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
-
-  if (quiet || verbose < verbose_level)
-    return;
-
-#if HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-  _doprnt (fmt, args, stderr);
-#endif
-
-  va_end (args);
-}
-
-
-void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  error (char *fmt, ...)
-#else
-  error (va_alist)
-     va_dcl
-#endif
-{
-  va_list args;
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start (args, fmt);
-#else
-  char *fmt;
-
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
-
-  fprintf (stderr, "%s: ", program);
-
-#if HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-  _doprnt (fmt, args, stderr);
-#endif
-  va_end (args);
-
-  fprintf (stderr, "\n");
-  fflush (stderr);
-}
-
-
-void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  fatal (char *fmt, ...)
-#else
-  fatal (va_alist)
-     va_dcl
-#endif
-{
-  va_list args;
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start (args, fmt);
-#else
-  char *fmt;
-
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
-
-  fprintf (stderr, "%s: ", program);
-
-#if HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-  _doprnt (fmt, args, stderr);
-#endif
-  va_end (args);
-
-  fprintf (stderr, "\n");
-  fflush (stderr);
-
-  exit (1);
-}
-
 #define GET_TOKEN(from) (strtok ((from), " \t\n"))
 #define GET_LINE_TOKEN(from) (strtok ((from), "\n"))
- 
+
 #define CHECK_TOKEN() 							\
   if (token2 == NULL) 							\
-    cfg_fatal (fname, line, _("missing argument: %s"), token);
+    CFG_FATAL ((stderr, _("missing argument: %s"), token));
 
 int
 read_config (char *path, char *file)
@@ -205,7 +114,7 @@ read_config (char *path, char *file)
 
       if (buf[0] == '#')
 	continue;
-      
+
       token = GET_TOKEN (buf);
       if (token == NULL)
 	/* Empty line. */
@@ -265,8 +174,8 @@ read_config (char *path, char *file)
 	  else if (MATCH (token2, "stdout"))
 	    output_file = OUTPUT_FILE_STDOUT;
 	  else
-	    cfg_fatal (fname, line, _("illegal value \"%s\" for option %s"),
-		       token2, token);
+	    CFG_FATAL ((stderr, _("illegal value \"%s\" for option %s"),
+			token2, token));
 	}
       else if (MATCH (token, "DownloadFont:"))
 	{
@@ -281,8 +190,8 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  escape_char = atoi (token2);
 	  if (escape_char < 0 || escape_char > 255)
-	    cfg_fatal (fname, line, _("invalid value \"%s\" for option %s"),
-		       token2, token);
+	    CFG_FATAL ((stderr, _("invalid value \"%s\" for option %s"),
+			token2, token));
 	}
       else if (MATCH (token, "FormFeedType:"))
 	{
@@ -293,8 +202,14 @@ read_config (char *path, char *file)
 	  else if (MATCH (token2, "page"))
 	    formfeed_type = FORMFEED_PAGE;
 	  else
-	    cfg_fatal (fname, line, _("illegal value \"%s\" for option %s"),
-		       token2, token);
+	    CFG_FATAL ((stderr, _("illegal value \"%s\" for option %s"),
+			token2, token));
+	}
+      else if (MATCH (token, "GeneratePageSize:"))
+	{
+	  token2 = GET_TOKEN (NULL);
+	  CHECK_TOKEN ();
+	  generate_PageSize = atoi (token2);
 	}
       else if (MATCH (token, "HighlightBarGray:"))
 	{
@@ -314,6 +229,12 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  strcpy (libpath, token2);
 	}
+      else if (MATCH (token, "MarkWrappedLines:"))
+	{
+	  token2 = GET_TOKEN (NULL);
+	  CHECK_TOKEN ();
+	  strcpy (mark_wrapped_lines_style_name, token2);
+	}
       else if (MATCH (token, "Media:"))
 	{
 	  char *name;
@@ -396,7 +317,7 @@ read_config (char *path, char *file)
 	}
       else if (MATCH (token, "SetPageDevice:"))
 	{
-	  token2 = GET_TOKEN (NULL);
+	  token2 = GET_LINE_TOKEN (NULL);
 	  CHECK_TOKEN ();
 	  parse_key_value_pair (pagedevice, token2);
 	}
@@ -424,6 +345,12 @@ read_config (char *path, char *file)
 	  CHECK_TOKEN ();
 	  strcpy (states_highlight_level, token2);
 	}
+      else if (MATCH (token, "StatesPath:"))
+	{
+	  token2 = GET_LINE_TOKEN (NULL);
+	  CHECK_TOKEN ();
+	  strcpy (states_path, token2);
+	}
       else if (MATCH (token, "StatusDict:"))
 	{
 	  token2 = GET_TOKEN (NULL);
@@ -455,7 +382,7 @@ read_config (char *path, char *file)
 	  token2 = GET_TOKEN (NULL);
 	  CHECK_TOKEN ();
 	  if (!parse_font_spec (token2, &ul_font, &ul_ptsize))
-	    cfg_fatal (fname, line, _("malformed font spec: %s"), token2);
+	    CFG_FATAL ((stderr, _("malformed font spec: %s"), token2));
 	}
       else if (MATCH (token, "UnderlayGray:"))
 	{
@@ -479,7 +406,7 @@ read_config (char *path, char *file)
 	  ul_style_str = ul_style_str_buf;
 	}
       else
-	cfg_fatal (fname, line, _("illegal option: %s"), token);
+	CFG_FATAL ((stderr, _("illegal option: %s"), token));
     }
   return 1;
 }
@@ -490,9 +417,10 @@ add_media (char *name, int w, int h, int llx, int lly, int urx, int ury)
 {
   MediaEntry *entry;
 
-  message (2,
-	   "add_media: name=%s, w=%d, h=%d, llx=%d, lly=%d, urx=%d, ury=%d\n",
-	   name, w, h, llx, lly, urx, ury);
+  MESSAGE (2,
+	   (stderr,
+	    "add_media: name=%s, w=%d, h=%d, llx=%d, lly=%d, urx=%d, ury=%d\n",
+	    name, w, h, llx, lly, urx, ury));
 
   entry = xcalloc (1, sizeof (*entry));
   entry->name = xmalloc (strlen (name) + 1);
@@ -642,7 +570,8 @@ paste_file (char *name, char *suffix)
 	  for (i = strlen (DIRECTIVE_FORMAT); buf[i] && isspace (buf[i]); i++)
 	    ;
 	  if (!buf[i])
-	    fatal (_("%s:%d: %%Format: no name"), ctx.fullname, line);
+	    FATAL ((stderr, _("%s:%d: %%Format: no name"), ctx.fullname,
+		    line));
 
 	  /* Copy name. */
 	  for (j = 0;
@@ -652,20 +581,20 @@ paste_file (char *name, char *suffix)
 	  name[j] = '\0';
 
 	  if (j >= sizeof (name) - 1)
-	    fatal (_("%s:%d: %%Format: too long name, maxlen=%d"),
-		   ctx.fullname, line, sizeof (name) - 1);
+	    FATAL ((stderr, _("%s:%d: %%Format: too long name, maxlen=%d"),
+		    ctx.fullname, line, sizeof (name) - 1));
 
 	  /* Find the start of the format string. */
 	  for (; buf[i] && isspace (buf[i]); i++)
 	    ;
-	  
+
 	  /* Find the end. */
 	  j = strlen (buf);
 	  for (j--; isspace (buf[j]) && j > i; j--)
 	    ;
 	  j++;
 
-	  message (2, "%%Format: %s %.*s\n", name, j - i, buf + i);
+	  MESSAGE (2, (stderr, "%%Format: %s %.*s\n", name, j - i, buf + i));
 
 	  cp = xmalloc (j - i + 1);
 	  memcpy (cp, buf + i, j - i);
@@ -674,8 +603,9 @@ paste_file (char *name, char *suffix)
 	  strhash_put (user_strings, name, strlen (name) + 1, cp,
 		       (void **) &cp2);
 	  if (cp2)
-	    fatal (_("%s:%d: %%Format: name \"%s\" is already defined"),
-		   ctx.fullname, line, name);
+	    FATAL ((stderr,
+		    _("%s:%d: %%Format: name \"%s\" is already defined"),
+		    ctx.fullname, line, name));
 
 	  /* All done with the `%Format' directive. */
 	  continue;
@@ -695,10 +625,11 @@ paste_file (char *name, char *suffix)
 		 buf[i] && !isspace (buf[i]); i++)
 	      ;
 	    if (!buf[i])
-	      fatal (_("%s:%d: %%HeaderHeight: no argument"), ctx.fullname, line);
+	      FATAL ((stderr, _("%s:%d: %%HeaderHeight: no argument"),
+		      ctx.fullname, line));
 
 	    d_header_h = atoi (buf + i);
-	    message (2, "%%HeaderHeight: %d\n", d_header_h);
+	    MESSAGE (2, (stderr, "%%HeaderHeight: %d\n", d_header_h));
 	    continue;
 	  }
 
@@ -716,10 +647,11 @@ paste_file (char *name, char *suffix)
 	       buf[i] && !isspace (buf[i]); i++)
 	    ;
 	  if (!buf[i])
-	    fatal (_("%s:%d: %%FooterHeight: no argument"), ctx.fullname, line);
+	    FATAL ((stderr, _("%s:%d: %%FooterHeight: no argument"),
+		    ctx.fullname, line));
 
 	  d_footer_h = atoi (buf + i);
-	  message (2, "%%FooterHeight: %d\n", d_footer_h);
+	  MESSAGE (2, (stderr, "%%FooterHeight: %d\n", d_footer_h));
 	  continue;
 	}
 
@@ -790,20 +722,21 @@ parse_font_spec (char *spec, char **name_return, FontPoint *size_return)
 	  size_return->w = size_return->h = atof (spec + i);
 	}
     }
-  
+
   *name_return = (char *) xcalloc (1, i + 1);
   strncpy (*name_return, spec, i);
 
-  message (2, "parse_font_spec(): name=%.*s, size=%g/%g\n", i, *name_return,
-	   size_return->w, size_return->h);
+  MESSAGE (2, (stderr, "parse_font_spec(): name=%.*s, size=%g/%g\n", i,
+	       *name_return, size_return->w, size_return->h));
 
   if (size_return->w < 0.0 && size_return->h < 0.0)
-    message (0, _("%s: warning: font size is negative\n"), program);
+    MESSAGE (0, (stderr, _("%s: warning: font size is negative\n"), program));
   else if (size_return->w < 0.0)
-    message (0, _("%s: warning: font width is negative\n"), program);
+    MESSAGE (0, (stderr, _("%s: warning: font width is negative\n"), program));
   else if (size_return->h < 0.0)
-    message (0, _("%s: warning: font height is negative\n"), program);
-    
+    MESSAGE (0, (stderr, _("%s: warning: font height is negative\n"),
+		 program));
+
   return 1;
 }
 
@@ -819,7 +752,7 @@ read_font_info (void)
   unsigned int enc_flags = 0;
   char fkey[256];
 
-  message (2, _("reading AFM info for font \"%s\"\n"), Fname);
+  MESSAGE (2, (stderr, _("reading AFM info for font \"%s\"\n"), Fname));
 
   if (accept_composites)
     enc_flags = AFM_ENCODE_ACCEPT_COMPOSITES;
@@ -840,38 +773,55 @@ read_font_info (void)
 	    {
 #define COUR "Courier"
 	      /*
-	       * Do not report failures for "Courier*" fonts because 
+	       * Do not report failures for "Courier*" fonts because
 	       * AFM library's default font will fix them.
 	       */
 	      if (strncmp (Fname, COUR, strlen (COUR)) != 0)
-		message (0, _("couldn't open AFM file for font \"%s\", using default\n"),
-			 Fname);
+		MESSAGE (0,
+			 (stderr,
+			  _("couldn't open AFM file for font \"%s\", using default\n"),
+			  Fname));
 	      error = afm_open_default_font (afm, &font);
 	      if (error != AFM_SUCCESS)
 		{
 		  afm_error_to_string (error, buf);
-		  fatal (_("couldn't open AFM file for the default font: %s"),
-			 buf);
+		  FATAL ((stderr,
+			  _("couldn't open AFM file for the default font: %s"),
+			  buf));
 		}
 	    }
 
 	  /* Apply encoding. */
 	  switch (encoding)
 	    {
-	    case ENC_LATIN1:
-	      (void) afm_font_encoding (font, AFM_ENCODING_LATIN1, enc_flags);
+	    case ENC_ISO_8859_1:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_1,
+					enc_flags);
 	      break;
 
-	    case ENC_LATIN2:
-	      (void) afm_font_encoding (font, AFM_ENCODING_LATIN2, enc_flags);
+	    case ENC_ISO_8859_2:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_2,
+					enc_flags);
 	      break;
 
-	    case ENC_LATIN3:
-	      (void) afm_font_encoding (font, AFM_ENCODING_LATIN3, enc_flags);
+	    case ENC_ISO_8859_3:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_3,
+					enc_flags);
 	      break;
 
-	    case ENC_LATIN5:
-	      (void) afm_font_encoding (font, AFM_ENCODING_LATIN5, enc_flags);
+	    case ENC_ISO_8859_4:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_4,
+					enc_flags);
+	      break;
+
+	    case ENC_ISO_8859_5:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_5,
+					enc_flags);
+	      break;
+
+	    case ENC_ISO_8859_7:
+	      (void) afm_font_encoding (font, AFM_ENCODING_ISO_8859_7,
+					enc_flags);
 	      break;
 
 	    case ENC_ASCII:
@@ -906,11 +856,11 @@ read_font_info (void)
 
 	    case ENC_MAC:
 	      (void) afm_font_encoding (font, AFM_ENCODING_MAC, enc_flags);
-	      break; 
+	      break;
 
 	    case ENC_VMS:
 	      (void) afm_font_encoding (font, AFM_ENCODING_VMS, enc_flags);
-	      break; 
+	      break;
 
 	    case ENC_HP8:
 	      (void) afm_font_encoding (font, AFM_ENCODING_HP8, enc_flags);
@@ -929,17 +879,16 @@ read_font_info (void)
 	  if (!strhash_put (afm_cache, Fname, strlen (Fname), font, NULL))
 	    font_cached = 0;
 	}
-      
+
       font_info = (CachedFontInfo *) xcalloc (1, sizeof (*font_info));
       /* Read character widths and types. */
       for (i = 0; i < 256; i++)
 	{
 	  AFMNumber w0x, w0y;
 
-	  (void) afm_font_charwidth (font, Fpt.w, (unsigned int) i,
-				     &w0x, &w0y);
+	  (void) afm_font_charwidth (font, Fpt.w, i, &w0x, &w0y);
 	  font_info->font_widths[i] = w0x;
-	      
+
 	  if (font->encoding[i] == AFM_ENC_NONE)
 	    font_info->font_ctype[i] = ' ';
 	  else if (font->encoding[i] == AFM_ENC_NON_EXISTENT)
@@ -1005,12 +954,13 @@ download_font (char *name)
 
   /* Ok, fine.  Font was found. */
 
-  message (1, _("downloading font \"%s\"\n"), name);
+  MESSAGE (1, (stderr, _("downloading font \"%s\"\n"), name));
   fp = fopen (fname, "rb");
   if (fp == NULL)
     {
-      message (0, _("couldn't open font description file \"%s\": %s\n"),
-	       fname, strerror (errno));
+      MESSAGE (0, (stderr,
+		   _("couldn't open font description file \"%s\": %s\n"),
+		   fname, strerror (errno)));
       return;
     }
 
@@ -1035,7 +985,7 @@ download_font (char *name)
       /* IBM PC Format */
 
       ungetc (i, fp);
-      
+
       while (!done)
 	{
 	  /* Read 6-byte long header. */
@@ -1296,8 +1246,9 @@ format_user_string (char *context_name, char *str)
 			   i++, j++)
 			buf2[j] = str[i];
 		      if (str[i] != '}')
-			fatal (_("%s: too long format for %%D{} escape"),
-			       context_name);
+			FATAL ((stderr,
+				_("%s: too long format for %%D{} escape"),
+				context_name));
 
 		      buf2[j] = '\0';
 		      strftime (buf, sizeof (buf), buf2, &run_tm);
@@ -1305,14 +1256,14 @@ format_user_string (char *context_name, char *str)
 		  else
 		    {
 		      /* `%D' run date in `yy-mm-dd' format */
-		      sprintf (buf, "%02d-%02d-%02d", run_tm.tm_year,
+		      sprintf (buf, "%02d-%02d-%02d", run_tm.tm_year % 100,
 			       run_tm.tm_mon + 1, run_tm.tm_mday);
 		    }
 		  APPEND_STR (buf);
 		  break;
 
 		case 'E':	/* `%E' run date in `yy/mm/dd' format */
-		  sprintf (buf, "%02d/%02d/%02d", run_tm.tm_year,
+		  sprintf (buf, "%02d/%02d/%02d", run_tm.tm_year % 100,
 			   run_tm.tm_mon + 1, run_tm.tm_mday);
 		  APPEND_STR (buf);
 		  break;
@@ -1321,7 +1272,7 @@ format_user_string (char *context_name, char *str)
 		  sprintf (buf, "%d.%d.%d",
 			   run_tm.tm_mday,
 			   run_tm.tm_mon + 1,
-			   run_tm.tm_year+1900);
+			   run_tm.tm_year + 1900);
 		  APPEND_STR (buf);
 		  break;
 
@@ -1376,13 +1327,13 @@ format_user_string (char *context_name, char *str)
 
 		case 'W':	/* `%W' run date in `mm/dd/yy' format */
 		  sprintf (buf, "%02d/%02d/%02d", run_tm.tm_mon + 1,
-			   run_tm.tm_mday, run_tm.tm_year);
+			   run_tm.tm_mday, run_tm.tm_year % 100);
 		  APPEND_STR (buf);
 		  break;
 
 		default:
-		  fatal (_("%s: unknown `%%' escape `%c' (%d)"), context_name,
-			 str[i], str[i]);
+		  FATAL ((stderr, _("%s: unknown `%%' escape `%c' (%d)"),
+			  context_name, str[i], str[i]));
 		  break;
 		}
 	    }
@@ -1396,7 +1347,10 @@ format_user_string (char *context_name, char *str)
 		  break;
 
 		case '%':	/* `$%' current page number */
-		  sprintf (buf, "%d", current_pagenum);
+		  if (slicing)
+		    sprintf (buf, "%d%c", current_pagenum, slice - 1 + 'A');
+		  else
+		    sprintf (buf, "%d", current_pagenum);
 		  APPEND_STR (buf);
 		  break;
 
@@ -1411,11 +1365,11 @@ format_user_string (char *context_name, char *str)
 		    buf[j++] = str[i];
 
 		  if (str[i] == '\0')
-		    fatal (_("%s: no closing ')' for $() escape"),
-			   context_name);
+		    FATAL ((stderr, _("%s: no closing ')' for $() escape"),
+			    context_name));
 		  if (str[i] != ')')
-		    fatal (_("%s: too long variable name for $() escape"),
-			   context_name);
+		    FATAL ((stderr, _("%s: too long variable name for $() escape"),
+			    context_name));
 
 		  buf[j] = '\0';
 
@@ -1440,8 +1394,9 @@ format_user_string (char *context_name, char *str)
 			   i++, j++)
 			buf2[j] = str[i];
 		      if (str[i] != '}')
-			fatal (_("%s: too long format for $D{} escape"),
-			       context_name);
+			FATAL ((stderr,
+				_("%s: too long format for $D{} escape"),
+				context_name));
 
 		      buf2[j] = '\0';
 		      strftime (buf, sizeof (buf), buf2, &mod_tm);
@@ -1449,14 +1404,14 @@ format_user_string (char *context_name, char *str)
 		  else
 		    {
 		      /* `$D' mod date in `yy-mm-dd' format */
-		      sprintf (buf, "%02d-%02d-%02d", mod_tm.tm_year,
+		      sprintf (buf, "%02d-%02d-%02d", mod_tm.tm_year % 100,
 			       mod_tm.tm_mon + 1, mod_tm.tm_mday);
 		    }
 		  APPEND_STR (buf);
 		  break;
 
 		case 'E':	/* `$E' mod date in `yy/mm/dd' format */
-		  sprintf (buf, "%02d/%02d/%02d", mod_tm.tm_year,
+		  sprintf (buf, "%02d/%02d/%02d", mod_tm.tm_year % 100,
 			   mod_tm.tm_mon + 1, mod_tm.tm_mday);
 		  APPEND_STR (buf);
 		  break;
@@ -1465,7 +1420,7 @@ format_user_string (char *context_name, char *str)
 		  sprintf (buf, "%d.%d.%d",
 			   mod_tm.tm_mday,
 			   mod_tm.tm_mon + 1,
-			   mod_tm.tm_year+1900);
+			   mod_tm.tm_year + 1900);
 		  APPEND_STR (buf);
 		  break;
 
@@ -1504,7 +1459,7 @@ format_user_string (char *context_name, char *str)
 
 		case 'W':	/* `$W' run date in `mm/dd/yy' format */
 		  sprintf (buf, "%02d/%02d/%02d", mod_tm.tm_mon + 1,
-			   mod_tm.tm_mday, mod_tm.tm_year);
+			   mod_tm.tm_mday, mod_tm.tm_year % 100);
 		  APPEND_STR (buf);
 		  break;
 
@@ -1526,8 +1481,8 @@ format_user_string (char *context_name, char *str)
 		  break;
 
 		default:
-		  fatal (_("%s: unknown `$' escape `%c' (%d)"), context_name,
-			 str[i], str[i]);
+		  FATAL ((stderr, _("%s: unknown `$' escape `%c' (%d)"),
+			  context_name, str[i], str[i]));
 		  break;
 		}
 	    }
@@ -1539,7 +1494,7 @@ format_user_string (char *context_name, char *str)
 	APPEND_CH (str[i]);
     }
   APPEND_CH ('\0');
-  
+
   /* Escape PS specials. */
   cp = escape_string (rbuf);
   xfree (rbuf);
@@ -1577,7 +1532,7 @@ count_key_value_set (StringHashPtr set)
   int i = 0, got, j;
   char *cp;
   void *value;
-  
+
   for (got = strhash_get_first (set, &cp, &j, &value); got;
        got = strhash_get_next (set, &cp, &j, &value))
     i++;
@@ -1593,7 +1548,7 @@ pathwalk (char *path, PathWalkProc proc, void *context)
   char *cp;
   char *cp2;
   int len, i;
-  
+
   for (cp = path; cp; cp = strchr (cp, PATH_SEPARATOR))
     {
       if (cp != path)
@@ -1625,18 +1580,18 @@ file_lookup (char *path, void *context)
   struct stat stat_st;
   int i;
 
-  message (2, "file_lookup(): %s/%s%s\t", path, ctx->name,
-	   ctx->suffix);
+  MESSAGE (2, (stderr, "file_lookup(): %s/%s%s\t", path, ctx->name,
+	       ctx->suffix));
 
   len = strlen (path);
   if (len && path[len - 1] == '/')
     len--;
 
   sprintf (ctx->fullname, "%.*s/%s%s", len, path, ctx->name, ctx->suffix);
-  
+
   i = stat (ctx->fullname, &stat_st) == 0;
 
-  message (2, "#%c\n", i ? 't' : 'f');
+  MESSAGE (2, (stderr, "#%c\n", i ? 't' : 'f'));
   return i;
 }
 
@@ -1693,8 +1648,8 @@ parse_float (char *string, int units, int horizontal)
   val = strtod (string, &end);
   if (end == string)
   malformed_float:
-    error (_("malformed float dimension: \"%s\""), string);
-  
+    ERROR ((stderr, _("malformed float dimension: \"%s\""), string));
+
   if (units)
     {
       switch (*end)
@@ -1746,7 +1701,9 @@ is_open (InputStream *is, FILE *fp, char *fname, char *input_filter)
   is->data_in_buf = 0;
   is->bufpos = 0;
   is->nreads = 0;
-  is->unget_ch = EOF;
+  is->unget_ch = NULL;
+  is->unget_pos = 0;
+  is->unget_alloc = 0;
 
   /* Input filter? */
   if (input_filter)
@@ -1766,7 +1723,7 @@ is_open (InputStream *is, FILE *fp, char *fname, char *input_filter)
        */
       cmdlen = strlen (input_filter) + 1;
       cmd = xmalloc (cmdlen);
-      
+
       /* Create filter command. */
       pos = 0;
       for (i = 0; input_filter[i]; i++)
@@ -1807,9 +1764,10 @@ is_open (InputStream *is, FILE *fp, char *fname, char *input_filter)
 
       if (is->fp == NULL)
 	{
-	  error (_("couldn't open input filter \"%s\" for file \"%s\": %s"),
-		 input_filter, fname ? fname : "(stdin)",
-		 strerror (errno));
+	  ERROR ((stderr,
+		  _("couldn't open input filter \"%s\" for file \"%s\": %s"),
+		  input_filter, fname ? fname : "(stdin)",
+		  strerror (errno)));
 	  return 0;
 	}
     }
@@ -1824,8 +1782,8 @@ is_open (InputStream *is, FILE *fp, char *fname, char *input_filter)
 	  is->fp = fopen (fname, "rb");
 	  if (is->fp == NULL)
 	    {
-	      error (_("couldn't open input file \"%s\": %s"), fname,
-		     strerror (errno));
+	      ERROR ((stderr, _("couldn't open input file \"%s\": %s"), fname,
+		      strerror (errno)));
 	      return 0;
 	    }
 	}
@@ -1842,6 +1800,9 @@ is_close (InputStream *is)
     pclose (is->fp);
   else
     fclose (is->fp);
+
+  if (is->unget_ch)
+    xfree (is->unget_ch);
 }
 
 
@@ -1850,10 +1811,9 @@ is_getc (InputStream *is)
 {
   int ch;
 
-  if (is->unget_ch != EOF)
+  if (is->unget_pos > 0)
     {
-      ch = is->unget_ch;
-      is->unget_ch = EOF;
+      ch = is->unget_ch[--is->unget_pos];
       return ch;
     }
 
@@ -1882,46 +1842,13 @@ is_getc (InputStream *is)
 int
 is_ungetc (int ch, InputStream *is)
 {
-  is->unget_ch = ch;
+  if (is->unget_pos >= is->unget_alloc)
+    {
+      is->unget_alloc += 1024;
+      is->unget_ch = xrealloc (is->unget_ch, is->unget_alloc);
+    }
+
+  is->unget_ch[is->unget_pos++] = ch;
+
   return 1;
 }
-
-
-/*
- * Static functions.
- */
-
-static void
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  cfg_fatal (char *file, int line, char *fmt, ...)
-#else
-  cfg_fatal (file, line, va_alist)
-     char *file;
-     int line;
-     va_dcl
-#endif
-{
-  va_list args;
-#if defined(HAVE_STDARG_H) && defined(__STDC__) && __STDC__
-  va_start (args, fmt);
-#else
-  char *fmt;
-
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
-
-  fprintf (stderr, "%s:%s:%d: ", program, file, line);
-
-#ifdef HAVE_VPRINTF
-  vfprintf (stderr, fmt, args);
-#else
-  _doprnt (fmt, args, stderr);
-#endif
-  va_end (args);
-
-  fprintf (stderr, "\n");
-  fflush (stderr);
-
-  exit (1);
-}
