@@ -1,4 +1,4 @@
-/* 
+/*
  * The main for states.
  * Copyright (c) 1997 Markku Rossi.
  *
@@ -7,7 +7,7 @@
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -60,6 +60,7 @@ unsigned int data_in_buffer;
 unsigned int bufpos;
 int eof_seen;
 char *current_fname;
+unsigned int current_linenum;
 
 struct re_registers *current_match = NULL;
 char *current_match_buf = NULL;
@@ -78,7 +79,7 @@ VariableDef *vardefs_tail = NULL;
 /*
  * -f NAME, --file=NAME
  *
- * Read state definitions from file NAME.  The default is "states.st" in 
+ * Read state definitions from file NAME.  The default is "states.st" in
  * the current working directory.
  */
 
@@ -91,7 +92,7 @@ unsigned int linenum = 1;
  * Save output to file FILE, the default is stdout.
  */
 
-FILE *ofp = stdout;
+FILE *ofp = NULL;
 
 /*
  * -s NAME, --state=NAME
@@ -150,6 +151,9 @@ main (argc, argv)
   int c;
   VariableDef *vardef;
 
+  /* Set defaults for options. */
+  ofp = stdout;
+
   /* Get program's name. */
   program = strrchr (argv[0], '/');
   if (program == NULL)
@@ -164,7 +168,7 @@ main (argc, argv)
   sprintf (version, _("states for GNU %s %s"), PACKAGE, VERSION);
 
   /* Internationalization. */
-#if HAVE_SETLOCALE 
+#if HAVE_SETLOCALE
 #if HAVE_LC_MESSAGES
   setlocale (LC_MESSAGES, "");
 #endif
@@ -190,7 +194,7 @@ main (argc, argv)
 
   init_primitives ();
 
-  re_set_syntax (RE_SYNTAX_AWK);
+  re_set_syntax (RE_SYNTAX_GNU_AWK);
 
   /* Enter some system variables. */
   enter_system_variable ("program", program);
@@ -203,7 +207,7 @@ main (argc, argv)
 
       c = getopt_long (argc, argv, "D:f:ho:s:VW:", long_options,
 		       &option_index);
-      if (c == EOF)
+      if (c == -1)
 	break;
 
       switch (c)
@@ -289,6 +293,32 @@ main (argc, argv)
 	}
     }
 
+  /* Pass all remaining arguments to States through `argv' array. */
+  {
+    Node *v, *n;
+    int i;
+
+    v = node_alloc (nARRAY);
+    v->u.array.allocated = argc - optind + 1;
+    v->u.array.len = argc - optind;
+    v->u.array.array = (Node **) xcalloc (v->u.array.allocated,
+					  sizeof (Node *));
+    for (i = optind; i < argc; i++)
+      {
+	n = node_alloc (nSTRING);
+	n->u.str.len = strlen (argv[i]);
+	n->u.str.data = xstrdup (argv[i]);
+	v->u.array.array[i - optind] = n;
+      }
+
+    if (!strhash_put (ns_vars, "argv", 4, v, (void **) &n))
+      {
+	fprintf (stderr, _("%s: out of memory\n"), program);
+	exit (1);
+      }
+    node_free (n);
+  }
+
   /* Parse config file. */
 
   yyin = fopen (defs_file, "r");
@@ -319,8 +349,7 @@ main (argc, argv)
 
       val = node_alloc (nSTRING);
       val->u.str.len = strlen (vardef->val);
-      val->u.str.data = (char *) xmalloc (val->u.str.len);
-      memcpy (val->u.str.data, vardef->val, val->u.str.len);
+      val->u.str.data = xstrdup (vardef->val);
 
       if (!strhash_put (ns_vars, vardef->sym, strlen (vardef->sym),
 			val, (void **) &old_val))
@@ -359,7 +388,7 @@ main (argc, argv)
 }
 
 
-/* 
+/*
  * Static functions.
  */
 
@@ -374,7 +403,7 @@ Mandatory arguments to long options are mandatory for short options too.\n"),
   -D, --define=VAR=VAL       define variable VAR to have value VAR\n\
   -f, --file=NAME            read state definitions from file NAME\n\
   -h, --help                 print this help and exit\n\
-  -o, --output=NAME          save output file file NAME\n\
+  -o, --output=NAME          save output to file NAME\n\
   -s, --state=NAME           start from state NAME\n\
   -V, --version              print version number\n\
   -W, --warning=LEVEL        set the warning level to LEVEL\n"));
