@@ -1,4 +1,4 @@
-/* 
+/*
  * General helper utilities.
  * Copyright (c) 1997 Markku Rossi.
  *
@@ -7,7 +7,7 @@
 
 /*
  * This file is part of GNU enscript.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -26,6 +26,17 @@
 
 #include "defs.h"
 
+/*
+ * Static variables.
+ */
+
+static RE_TRANSLATE_TYPE case_insensitive_translate = NULL;
+
+
+/*
+ * Global functions.
+ */
+
 /* Generic linked list. */
 
 List *
@@ -83,11 +94,69 @@ node_alloc (type)
   n = (Node *) xcalloc (1, sizeof (*n));
   n->type = type;
   n->refcount = 1;
+  n->linenum = linenum;
+
+  if (type == nREGEXP)
+    n->u.re.compiled.fastmap = xmalloc (256);
 
   return n;
 }
 
 
+Node *
+node_copy (n)
+     Node *n;
+{
+  Node *n2;
+  int i;
+
+  n2 = node_alloc (n->type);
+  n2->linenum = n->linenum;
+
+  switch (n->type)
+    {
+    case nVOID:
+      /* All done. */
+      break;
+
+    case nSTRING:
+      n2->u.str.len = n->u.str.len;
+      /* +1 to avoid zero allocation. */
+      n2->u.str.data = (char *) xmalloc (n2->u.str.len + 1);
+      memcpy (n2->u.str.data, n->u.str.data, n->u.str.len);
+      break;
+
+    case nREGEXP:
+      n2->u.re.data = xstrdup (n->u.re.data);
+      n2->u.re.len = n->u.re.len;
+      break;
+
+    case nINTEGER:
+      n2->u.integer = n->u.integer;
+      break;
+
+    case nREAL:
+      n2->u.real = n->u.real;
+      break;
+
+    case nSYMBOL:
+      n2->u.sym = xstrdup (n->u.sym);
+      break;
+
+    case nARRAY:
+      n2->u.array.len = n->u.array.len;
+      n2->u.array.allocated = n2->u.array.len + 1;
+      n2->u.array.array = (Node **) xcalloc (n2->u.array.allocated,
+					     sizeof (Node *));
+      for (i = 0; i < n->u.array.len; i++)
+	n2->u.array.array[i] = node_copy (n->u.array.array[i]);
+      break;
+    }
+
+  return n2;
+}
+
+
 void
 node_reference (node)
      Node *node;
@@ -121,7 +190,8 @@ node_free (node)
       break;
 
     case nREGEXP:
-      xfree (node->u.re.data);
+      free (node->u.re.data);
+      xfree (node->u.re.compiled.fastmap);
       break;
 
     case nINTEGER:
@@ -161,6 +231,41 @@ enter_system_variable (name, value)
 }
 
 
+void
+compile_regexp (re)
+     Node *re;
+{
+  const char *msg;
+
+  if (case_insensitive_translate == NULL)
+    {
+      int i;
+
+      case_insensitive_translate = xmalloc (256);
+
+      for (i = 0; i < 256; i++)
+	if (isupper (i))
+	  case_insensitive_translate[i] = tolower (i);
+	else
+	  case_insensitive_translate[i] = i;
+    }
+
+  if (re->u.re.flags & fRE_CASE_INSENSITIVE)
+    re->u.re.compiled.translate = case_insensitive_translate;
+
+  msg = re_compile_pattern (re->u.re.data, re->u.re.len, &re->u.re.compiled);
+  if (msg)
+    {
+      fprintf (stderr,
+	       _("%s:%d: couldn't compile regular expression \"%s\": %s\n"),
+	       defs_file, re->linenum, re->u.re.data, msg);
+      exit (1);
+    }
+
+  re_compile_fastmap (&re->u.re.compiled);
+}
+
+
 /*
  * Grammar constructors.
  */
@@ -244,17 +349,28 @@ mk_expr (type, arg1, arg2, arg3)
     case eNOT:
       expr->u.not = arg1;
       break;
-      
+
     case eFCALL:
       expr->u.fcall.name = arg1;
       expr->u.fcall.args = arg2;
       break;
 
     case eASSIGN:
+    case eADDASSIGN:
+    case eSUBASSIGN:
+    case eMULASSIGN:
+    case eDIVASSIGN:
       expr->u.assign.sym = arg1;
       expr->u.assign.expr = arg2;
       break;
 
+    case ePOSTFIXADD:
+    case ePOSTFIXSUB:
+    case ePREFIXADD:
+    case ePREFIXSUB:
+      expr->u.node = arg1;
+      break;
+
     case eARRAYASSIGN:
       expr->u.arrayassign.expr1 = arg1;
       expr->u.arrayassign.expr2 = arg2;
@@ -308,7 +424,7 @@ cons (car, cdr)
 }
 
 
-void 
+void
 define_state (sym, rules)
      Node *sym;
      List *rules;
@@ -353,61 +469,288 @@ define_sub (sym, args_body, linenum)
 	     defs_file, linenum, sym->u.sym);
 }
 
+extern unsigned int current_linenum;
 
 static Node *
-lookup_var (env, ns, sym)
+lookup_var (env, ns, sym, linenum)
      Environment *env;
      StringHashPtr ns;
      Node *sym;
+     unsigned int linenum;
 {
   Node *n;
+  Environment *e;
 
-  /* Regexp sub expression reference. */
-  if (sym->u.sym[0] == '$' 
-      && sym->u.sym[1] >= '0' && sym->u.sym[1] <= '9'
-      && sym->u.sym[2] == '\0')
+  /* Special variables. */
+  if (sym->u.sym[0] == '$' && sym->u.sym[1] && sym->u.sym[2] == '\0')
     {
-      int i;
-      int len;
-
-      /* Matched text. */
-      i = sym->u.sym[1] - '0';
-
-      n = node_alloc (nSTRING);
-      if (current_match == NULL || current_match->start[i] < 0
-	  || current_match_buf == NULL)
+      /* Regexp sub expression reference. */
+      if (sym->u.sym[1] >= '0' && sym->u.sym[1] <= '9')
 	{
-	  n->u.str.data = (char *) xmalloc (1);
-	  n->u.str.len = 0;
+	  int i;
+	  int len;
+
+	  /* Matched text. */
+	  i = sym->u.sym[1] - '0';
+
+	  n = node_alloc (nSTRING);
+	  if (current_match == NULL || current_match->start[i] < 0
+	      || current_match_buf == NULL)
+	    {
+	      n->u.str.data = (char *) xmalloc (1);
+	      n->u.str.len = 0;
+	    }
+	  else
+	    {
+	      len = current_match->end[i] - current_match->start[i];
+	      n->u.str.data = (char *) xmalloc (len + 1);
+	      memcpy (n->u.str.data,
+		      current_match_buf + current_match->start[i], len);
+	      n->u.str.len = len;
+	    }
+
+	  return n;
+	}
+
+      /* Everything before the matched expression. */
+      if (sym->u.sym[1] == '`' || sym->u.sym[1] == 'B')
+	{
+	  n = node_alloc (nSTRING);
+	  if (current_match == NULL || current_match->start[0] < 0
+	      || current_match_buf == NULL)
+	    {
+	      n->u.str.data = (char *) xmalloc (1);
+	      n->u.str.len = 0;
+	    }
+	  else
+	    {
+	      n->u.str.len = current_match->start[0];
+	      n->u.str.data = (char *) xmalloc (n->u.str.len + 1);
+	      memcpy (n->u.str.data, current_match_buf, n->u.str.len);
+	    }
+
+	  return n;
+	}
+
+      /* Current input line number. */
+      if (sym->u.sym[1] == '.')
+	{
+	  n = node_alloc (nINTEGER);
+	  n->u.integer = current_linenum;
+	  return n;
+	}
+    }
+
+  /* Local variables. */
+  for (e = env; e; e = e->next)
+    if (strcmp (e->name, sym->u.sym) == 0)
+      return e->val;
+
+  /* Global variables. */
+  if (strhash_get (ns, sym->u.sym, strlen (sym->u.sym), (void **) &n))
+    return n;
+
+  /* Undefined variable. */
+  fprintf (stderr, _("%s:%d: error: undefined variable `%s'\n"),
+	   defs_file, linenum, sym->u.sym);
+  exit (1);
+
+  /* NOTREACHED */
+  return NULL;
+}
+
+
+static void
+set_var (env, ns, sym, val, linenum)
+     Environment *env;
+     StringHashPtr ns;
+     Node *sym;
+     Node *val;
+     unsigned int linenum;
+{
+  Node *n;
+  Environment *e;
+
+  /* Local variables. */
+  for (e = env; e; e = e->next)
+    if (strcmp (e->name, sym->u.sym) == 0)
+      {
+	node_free (e->val);
+	e->val = val;
+	return;
+      }
+
+  /* Global variables. */
+  if (strhash_put (ns, sym->u.sym, strlen (sym->u.sym), val, (void **) &n))
+    {
+      node_free (n);
+      return;
+    }
+
+  /* Couldn't set value for variable. */
+  fprintf (stderr, _("%s:%d: error: couldn't set variable `%s'\n"),
+	   defs_file, linenum, sym->u.sym);
+  exit (1);
+  /* NOTREACHED */
+}
+
+
+static Node *
+calculate_binary (l, r, type, linenum)
+     Node *l;
+     Node *r;
+     ExprType type;
+     unsigned int linenum;
+{
+  Node *n = NULL;
+
+  switch (type)
+    {
+    case eMULT:
+    case eDIV:
+    case ePLUS:
+    case eMINUS:
+    case eLT:
+    case eGT:
+    case eEQ:
+    case eNE:
+    case eGE:
+    case eLE:
+      if (l->type == r->type && l->type == nINTEGER)
+	{
+	  n = node_alloc (nINTEGER);
+	  switch (type)
+	    {
+	    case eMULT:
+	      n->u.integer = (l->u.integer * r->u.integer);
+	      break;
+
+	    case eDIV:
+	      n->u.integer = (l->u.integer / r->u.integer);
+	      break;
+
+	    case ePLUS:
+	      n->u.integer = (l->u.integer + r->u.integer);
+	      break;
+
+	    case eMINUS:
+	      n->u.integer = (l->u.integer - r->u.integer);
+	      break;
+
+	    case eLT:
+	      n->u.integer = (l->u.integer < r->u.integer);
+	      break;
+
+	    case eGT:
+	      n->u.integer = (l->u.integer > r->u.integer);
+	      break;
+
+	    case eEQ:
+	      n->u.integer = (l->u.integer == r->u.integer);
+	      break;
+
+	    case eNE:
+	      n->u.integer = (l->u.integer != r->u.integer);
+	      break;
+
+	    case eGE:
+	      n->u.integer = (l->u.integer >= r->u.integer);
+	      break;
+
+	    case eLE:
+	      n->u.integer = (l->u.integer <= r->u.integer);
+	      break;
+
+	    default:
+	      /* NOTREACHED */
+	      break;
+	    }
+	}
+      else if ((l->type == nINTEGER || l->type == nREAL)
+	       && (r->type == nINTEGER || r->type == nREAL))
+	{
+	  double dl, dr;
+
+	  if (l->type == nINTEGER)
+	    dl = (double) l->u.integer;
+	  else
+	    dl = l->u.real;
+
+	  if (r->type == nINTEGER)
+	    dr = (double) r->u.integer;
+	  else
+	    dr = r->u.real;
+
+	  n = node_alloc (nREAL);
+	  switch (type)
+	    {
+	    case eMULT:
+	      n->u.real = (dl * dr);
+	      break;
+
+	    case eDIV:
+	      n->u.real = (dl / dr);
+	      break;
+
+	    case ePLUS:
+	      n->u.real = (dl + dr);
+	      break;
+
+	    case eMINUS:
+	      n->u.real = (dl - dr);
+	      break;
+
+	    case eLT:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl < dr);
+	      break;
+
+	    case eGT:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl > dr);
+	      break;
+
+	    case eEQ:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl == dr);
+	      break;
+
+	    case eNE:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl != dr);
+	      break;
+
+	    case eGE:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl >= dr);
+	      break;
+
+	    case eLE:
+	      n->type = nINTEGER;
+	      n->u.integer = (dl <= dr);
+	      break;
+
+	    default:
+	      /* NOTREACHED */
+	      break;
+	    }
 	}
       else
 	{
-	  len = current_match->end[i] - current_match->start[i];
-	  n->u.str.data = (char *) xmalloc (len + 1);
-	  memcpy (n->u.str.data, current_match_buf + current_match->start[i],
-		  len);
-	  n->u.str.len = len;
+	  fprintf (stderr,
+		   _("%s:%d: error: expression between illegal types\n"),
+		   defs_file, linenum);
+	  exit (1);
 	}
+      break;
 
-      return n;
+    default:
+      /* This is definitely a bug. */
+      abort ();
+      break;
     }
 
-  /* Local and global variables. */
-  if (env)
-    {
-      Environment *e;
-
-      for (e = env; e; e = e->next)
-	if (strcmp (e->name, sym->u.sym) == 0)
-	  return e->val;
-    }
-  else if (ns)
-    {
-      if (strhash_get (ns, sym->u.sym, strlen (sym->u.sym), (void **) &n))
-	return n;
-    }
-
-  return NULL;
+  return n;
 }
 
 
@@ -424,6 +767,7 @@ eval_expr (expr, env)
   int return_seen;
   Environment *ei, *ei2;
   int i;
+  Node sn;
 
   if (expr == NULL)
     return nvoid;
@@ -439,18 +783,7 @@ eval_expr (expr, env)
       break;
 
     case eSYMBOL:
-      /* Current environment. */
-      n = lookup_var (env, NULL, expr->u.node);
-      if (n == NULL)
-	/* Global variables. */
-	n = lookup_var (NULL, ns_vars, expr->u.node);
-      if (n == NULL)
-	{
-	  fprintf (stderr, _("%s:%d: error: undefined variable `%s'\n"),
-		   defs_file, expr->linenum, expr->u.node->u.sym);
-	  exit (1);
-	}
-
+      n = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
       node_reference (n);
       return n;
       break;
@@ -554,7 +887,7 @@ eval_expr (expr, env)
       else if (strhash_get (ns_prims, n->u.sym, strlen (n->u.sym),
 			    (void **) &prim))
 	{
-	  n = (*prim) (n->u.sym, expr->u.fcall.args, env);
+	  n = (*prim) (n->u.sym, expr->u.fcall.args, env, expr->linenum);
 	  return n;
 	}
       else
@@ -567,24 +900,75 @@ eval_expr (expr, env)
 
     case eASSIGN:
       n = eval_expr (expr->u.assign.expr, env);
+      set_var (env, ns_vars, expr->u.assign.sym, n, expr->linenum);
 
-      /* Local variable? */
-      for (ei = env; ei; ei = ei->next)
-	if (strcmp (ei->name, expr->u.node->u.sym) == 0)
+      node_reference (n);
+      return n;
+      break;
+
+    case eADDASSIGN:
+    case eSUBASSIGN:
+    case eMULASSIGN:
+    case eDIVASSIGN:
+      n = eval_expr (expr->u.assign.expr, env);
+      n2 = lookup_var (env, ns_vars, expr->u.assign.sym, expr->linenum);
+
+      switch (expr->type)
+	{
+	case eADDASSIGN:
+	  n2 = calculate_binary (n2, n, ePLUS, expr->linenum);
+	  break;
+
+	case eSUBASSIGN:
+	  n2 = calculate_binary (n2, n, eMINUS, expr->linenum);
+	  break;
+
+	case eMULASSIGN:
+	  n2 = calculate_binary (n2, n, eMULT, expr->linenum);
+	  break;
+
+	case eDIVASSIGN:
+	  n2 = calculate_binary (n2, n, eDIV, expr->linenum);
+	  break;
+
+	default:
+	  /* NOTREACHED */
+	  abort ();
 	  break;
-      if (ei)
-	{
-	  /* Yes, re-define it. */
-	  node_free (ei->val);
-	  ei->val = n;
-	}
-      else
-	{
-	  /* Global variable. */
-	  strhash_put (ns_vars, expr->u.node->u.sym,
-		       strlen (expr->u.node->u.sym), n, (void **) &n2);
-	  node_free (n2);
 	}
+      set_var (env, ns_vars, expr->u.assign.sym, n2, expr->linenum);
+
+      node_free (n);
+      node_reference (n2);
+      return n2;
+      break;
+
+    case ePOSTFIXADD:
+    case ePOSTFIXSUB:
+      sn.type = nINTEGER;
+      sn.u.integer = 1;
+
+      n2 = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
+      node_reference (n2);
+
+      n = calculate_binary (n2, &sn,
+			    expr->type == ePOSTFIXADD ? ePLUS : eMINUS,
+			    expr->linenum);
+      set_var (env, ns_vars, expr->u.node, n, expr->linenum);
+
+      return n2;
+      break;
+
+    case ePREFIXADD:
+    case ePREFIXSUB:
+      sn.type = nINTEGER;
+      sn.u.integer = 1;
+
+      n = lookup_var (env, ns_vars, expr->u.node, expr->linenum);
+      n = calculate_binary (n, &sn,
+			    expr->type == ePREFIXADD ? ePLUS : eMINUS,
+			    expr->linenum);
+      set_var (env, ns_vars, expr->u.node, n, expr->linenum);
 
       node_reference (n);
       return n;
@@ -659,7 +1043,7 @@ eval_expr (expr, env)
 	  l = eval_expr (expr->u.arrayassign.expr3, env);
 	  if (l->type != nINTEGER)
 	    {
-	      fprintf (stderr, 
+	      fprintf (stderr,
 		       _("%s:%d: error: illegal rvalue for string assignment\n"),
 		       defs_file, expr->linenum);
 	      exit (1);
@@ -710,7 +1094,8 @@ eval_expr (expr, env)
       else
 	{
 	  l = node_alloc (nINTEGER);
-	  l->u.integer = (int) ((unsigned char *) n->u.str.data)[n2->u.integer];
+	  l->u.integer
+	    = (int) ((unsigned char *) n->u.str.data)[n2->u.integer];
 	}
       node_free (n);
       node_free (n2);
@@ -761,133 +1146,14 @@ eval_expr (expr, env)
       l = eval_expr (expr->u.op.left, env);
       r = eval_expr (expr->u.op.right, env);
 
-      if (l->type == r->type && l->type == nINTEGER)
-	{
-	  n = node_alloc (nINTEGER);
-	  switch (expr->type)
-	    {
-	    case eMULT:
-	      n->u.integer = (l->u.integer * r->u.integer);
-	      break;
-
-	    case eDIV:
-	      n->u.integer = (l->u.integer / r->u.integer);
-	      break;
-
-	    case ePLUS:
-	      n->u.integer = (l->u.integer + r->u.integer);
-	      break;
-
-	    case eMINUS:
-	      n->u.integer = (l->u.integer - r->u.integer);
-	      break;
-
-	    case eLT:
-	      n->u.integer = (l->u.integer < r->u.integer);
-	      break;
-
-	    case eGT:
-	      n->u.integer = (l->u.integer > r->u.integer);
-	      break;
-
-	    case eEQ:
-	      n->u.integer = (l->u.integer == r->u.integer);
-	      break;
-
-	    case eNE:
-	      n->u.integer = (l->u.integer != r->u.integer);
-	      break;
-
-	    case eGE:
-	      n->u.integer = (l->u.integer >= r->u.integer);
-	      break;
-
-	    case eLE:
-	      n->u.integer = (l->u.integer <= r->u.integer);
-	      break;
-
-	    default:
-	      /* NOTREACHED */
-	      break;
-	    }
-	}
-      else if ((l->type == nINTEGER || l->type == nREAL)
-	       && (r->type == nINTEGER || r->type == nREAL))
-	{
-	  double dl, dr;
-
-	  if (l->type == nINTEGER)
-	    dl = (double) l->u.integer;
-	  else
-	    dl = l->u.real;
-
-	  if (r->type == nINTEGER)
-	    dr = (double) r->u.integer;
-	  else
-	    dr = r->u.real;
-
-	  n = node_alloc (nREAL);
-	  switch (expr->type)
-	    {
-	    case eMULT:
-	      n->u.real = (dl * dr);
-	      break;
-
-	    case eDIV:
-	      n->u.real = (dl / dr);
-	      break;
-
-	    case ePLUS:
-	      n->u.real = (dl + dr);
-	      break;
-
-	    case eMINUS:
-	      n->u.real = (dl - dr);
-	      break;
-
-	    case eLT:
-	      n->u.real = (dl < dr);
-	      break;
-
-	    case eGT:
-	      n->u.real = (dl > dr);
-	      break;
-
-	    case eEQ:
-	      n->u.real = (dl == dr);
-	      break;
-
-	    case eNE:
-	      n->u.real = (dl != dr);
-	      break;
-
-	    case eGE:
-	      n->u.real = (dl >= dr);
-	      break;
-
-	    case eLE:
-	      n->u.real = (dl <= dr);
-	      break;
-
-	    default:
-	      /* NOTREACHED */
-	      break;
-	    }
-	}
-      else
-	{
-	  fprintf (stderr,
-		   _("%s:%d: error: expression between illegal types\n"),
-		   defs_file, expr->linenum);
-	  exit (1);
-	}
+      n = calculate_binary (l, r, expr->type, expr->linenum);
 
       node_free (l);
       node_free (r);
       return n;
       break;
     }
-  
+
   /* NOTREACHED */
   return n;
 }
@@ -926,7 +1192,7 @@ eval_statement (stmt, env, return_seen)
       if (i)
 	/* Then branch. */
 	n = eval_statement (stmt->u.stmt_if.then_stmt, env, return_seen);
-      else 
+      else
 	{
 	  /* Optional else branch.  */
 	  if (stmt->u.stmt_if.else_stmt)
@@ -962,14 +1228,14 @@ eval_statement (stmt, env, return_seen)
 	  n2 = eval_expr (stmt->u.stmt_for.init, env);
 	  node_free (n2);
 	}
-      
+
       /* Body. */
       while (1)
 	{
 	  n2 = eval_expr (stmt->u.stmt_for.cond, env);
 	  i = IS_TRUE (n2);
 	  node_free (n2);
-	  
+
 	  if (!i)
 	    break;
 
